program cap(fin, fout, output);
(* put capital letters inside quotes of a program

  Dr. Thomas D. Schneider
  toms@alum.mit.edu
  https://alum.mit.edu/www/toms/

module libraries required: delman, delmods
*)

label 1; (* the end of the program *)

const
(* begin module version *)
   version = 1.09; (* of cap.p 2019 Apr 08
2019 Apr 08, 1.09; sin->fin and sout->fout for gcc.
1989 Jul 08, 1.08; functional
*)
(* end module version *)

(* begin module describe.cap *)
(*
name
      cap: put capital letters inside quotes of a program

synopsis
      cap(fin: in, fout: out,output: out)

files
      fin: the source program or file
      fout: the source program with capital letters in all
         quote strings.
      output: messages to the user

description
      A pascal program under Unix must be small characters, yet a
      database will often be in capital letters, so the program will
      not recognize the data.  This program makes the fin program
      have capital letters only in the quote strings.

author
      Thomas D. Schneider

bugs
      none known

*)
(* end module describe.cap *)

(* begin module cap.const *)
      single = ''''; (* a single quote mark *)
      double = '"'; (* a double quote mark *)
var
(* begin module cap.var *)
      fin, (* the source in file *)
      fout: (* the source out file *)
         text;
(* end module cap.var *)

(* begin module halt *)
procedure halt;
(* stop the program *)
begin
      goto 1
end;
(* begin module halt *)

(* begin module cap.docomment *)
procedure docomment(var fin,fout: text);
(* do a comment: find the end of the comment and copy all of it
from fin to fout *)
var
(*            endcomment := false; the eoln wrecks the potential comment *)
      ch: char; (* a character from fin *)
      done: boolean; (* when we have found the end of the comment *)
      endcomment: boolean; (* if true, perhaps we are at the end *)
begin
      endcomment := false;
      done := false;
      while (not eof(fin)) and (not done) do begin
         if eoln(fin)
         then begin
            readln(fin);
            writeln(fout);
            endcomment := false;
         end
         else begin
            read(fin,ch);
            write(fout,ch);

            if (ch = ')') and endcomment
            then done := true
            else endcomment := (ch = '*')
         end
      end
end;
(* end module cap.docomment *)

(* begin module cap.doquote *)
procedure doquote(var fin, fout: text; endquote: char;
                  diff: integer);
(* do a quote: copy from fin to fout until the endquote character
is found in fin.  all letters inside the quotes are made into
upper case, by subtracting the value of diff. *)
var
      ch: char; (* a character from fin *)
begin
      ch := ' ';
      while (not eof(fin)) and (ch<>endquote) do begin
         if eoln(fin)
         then begin
            readln(fin);
            writeln(fout);
         end
         else begin
            read(fin,ch);
            if (ch in ['a'..'z'])
            then write(fout,chr(ord(ch)-diff))
            else write(fout,ch)
         end
      end
end;
(* end module cap.doquote *)

(* begin module cap.main *)
procedure themain(var fin, fout: text);
(* the main procedure of the program *)
var
      ch: char; (* a character from fin *)
      diff: integer; (* the difference in ord between small and
         capital letters.  the value of diff will change from one
         computer system to another, but the program will still
         work the same way. *)
      startcomment: boolean; (* true when a comment may have started *)
begin
      writeln(output,' cap ',version:4:2);

      reset(fin);
      rewrite(fout);
      startcomment := false;
      diff := ord('a') - ord('A'); (* difference between small and caps *)

      while not eof(fin) do begin
         if eoln(fin)
         then begin
            readln(fin);
            writeln(fout);
            startcomment := false
         end
         else begin
            read(fin,ch);
            write(fout,ch);

            if (ch = '*') and startcomment
            then begin
               docomment(fin,fout);
               startcomment := false;
            end
            else begin
               if ch = '('
               then startcomment := true
               else begin
                  startcomment := false;
                  if (ch = single) or (ch = double)
                  then doquote(fin,fout,ch,diff)
               end
            end
         end
      end
end;
(* end module cap.main *)

begin
      themain(fin,fout);
1: end.
