program genmod(entries,output);
(* genbank access modules
   by thomas schneider, copyright (c) 1986
module libraries required: delman, prgmods *)
  
label 1; (* end of program *) 
  
const 
(* begin module version *)
version = 1.33; (* of genmod, 1986 feb 4
origin 1986 jan 27 *)
(* end module version *)
  
(* begin module describe.genmod *)
(*
name
      genmod: genbank access modules

synopsis
      genmod(entries: in, output: out) 
  
files 
      entries: a set of genbank entries for a given organism
      output: messages to the user and tests of the modules
  
description 
      these are modules containing procedures to access genbank
      entries.

author
      thomas d. schneider 
  
bugs
      none known  
  
*)
(* end module describe.genmod *)

(* begin module genbank.const.package *)
(* ********************************************************************** *)
(* begin module interact.const *) 
(* end module interact.const *) 

(* begin module genmod.filler.const *) 
(* end module genmod.filler.const *) 
(* ********************************************************************** *)
(* end module genbank.const.package *)

(* begin module genbank.type.package *)
(* ********************************************************************** *)
(* begin module interact.type *)
(* end module interact.type *)

(* begin module trigger.type *) 
(* end module trigger.type *) 

(* begin module filler.type *)
(* end module filler.type *)

(* ********************************************************************** *)
(* end module genbank.type.package *)

(* begin module genbank.string.package *)
(* ********************************************************************** *)
(* begin module interact.clearstring *) 
(* end module interact.clearstring *) 

(* begin module interact.getstring *)
(* end module interact.getstring *)

(* begin module interact.writestring *) 
(* end module interact.writestring *) 

(* begin module filler.fillstring *)
(* end module filler.fillstring *)

(* begin module trigger.proc *) 
(* end module trigger.proc *) 

(* begin module filler.filltrigger *) 
(* end module filler.filltrigger *) 
(* ********************************************************************** *)
(* end module genbank.string.package *)

(* begin module genbank.find.package *)
(* ********************************************************************** *)
(* begin module genbank.objonline *)
(* end module genbank.objonline *)

(* begin module genbank.find *)
(* end module genbank.find *)

(* begin module genbank.findat *)
(* end module genbank.findat *)
(* ********************************************************************** *)
(* end module genbank.find.package *)

(* begin module genbank.getbase.package *)
(* ********************************************************************** *)
(* begin module genbank.string.package *)
(* end module genbank.string.package *)

(* begin module genbank.find.package *)
(* end module genbank.find.package *)

(* begin module skipblanks *) 
(* end module skipblanks *) 

(* begin module genbank.findsequence *)
(* end module genbank.findsequence *)

(* begin module genbank.getbase *)
(* end module genbank.getbase *)
(* ********************************************************************** *)
(* end module genbank.getbase.package *)
  
(* begin module interact.const *) 
      maxstring = 150; (* the maximum string *) 
(* end module interact.const version = 'prgmod 3.97  85 may 5 tds'; *)
  
(* begin module genmod.filler.const *) 
      fillermax = 20; (* the size of the filler array for a string *) 
(* end module genmod.filler.const *)

type  
(* begin module interact.type *)
      string = record (* a string of characters *)  
         letters: array[1..maxstring] of char; (* the letters in the string *)
         length: integer; (* the number of characters in the string *)
         current: integer; (* the letter we are working on *) 
      end;  
(* end module interact.type version = 'prgmod 3.97  85 may 5 tds'; *)

(* begin module trigger.type *) 
      trigger = record (* an object to be searched for *) 
         seek: string; (* the characters looked for *)  
         state: integer; (* how close to triggering we are *) 
         skip: boolean; (* trigger not found- skip the line *)
         found: boolean (* the trigger was found *) 
      end;  
(* end module trigger.type version = 'prgmod 3.97  85 may 5 tds'; *)

(* begin module filler.type *)
      (* the following is an array used to fill a string. 
      it is convenient to have it much shorter than the maxstring, so that  
      it is easy to fill the string using procedure fillstring. 
      the user must declare the value of constant fillermax. *) 
      filler = packed array[1..fillermax] of char;
(* end module filler.type version = 'prgmod 3.97  85 may 5 tds'; *)
  
var 
(* begin module genmod.var *)
      entries: (* a set of genbank entries *)
         text;
(* end module genmod.var *)

(* begin module halt *) 
procedure halt; 
(* stop the program.  the procedure performs a goto to the end of the   
   program.  you must have a label: 
      label 1;
   declared, and also the end of the program must have this label:  
      1: end. 
   examples are in the module libraries.
   this is the only goto in the delila system. *) 
begin 
      writeln(output,' program halt.'); 
      goto 1
end;  
(* end module halt version = 'prgmod 3.97  85 may 5 tds'; *)
  
(* begin module interact.clearstring *) 
procedure clearstring(var ribbon: string);  
(* empty the string *)
var   index: integer; (* to the ribbon *) 
begin (* clearstring *) 
      with ribbon do begin  
         for index := 1 to maxstring do letters[index] := ' ';
         length := 0; 
         current := 0;
      end 
end; (* clearstring *)
(* end module interact.clearstring version = 'prgmod 3.97  85 may 5 tds'; *)

(* begin module interact.getstring *)
procedure getstring(var afile: text; var buffer: string;  
                    var gotten: boolean); 
(* get a string from a file not using string calls.  this lets one  
obtain lines from a file without interactive prompts *) 
var   index: integer; (* of buffer *) 
begin (* getstring *) 
      clearstring(buffer);  
      if eof(afile) 
      then gotten := false  
      else begin  
         index := 0;
         while (not eoln(afile)) and (index < maxstring) do begin 
            index := succ(index); 
            read(afile, buffer.letters[index])
         end; 
  
         if not eoln(afile) then begin
            writeln(output, ' getstring: a line exceeds maximum string size (', 
                            maxstring:1,')'); 
            halt
         end; 
  
         buffer.length := index;
         buffer.current := 1; 
         readln(afile); 
         gotten := true 
      end 
end; (* getstring *)
(* end module interact.getstring version = 'prgmod 3.97  85 may 5 tds'; *)

(* begin module interact.writestring *) 
procedure writestring(var tofile: text; var s: string); 
(* write the string s to file tofile, no writeln *) 
var   i: integer; (* index to s *)  
begin (* writestring *) 
      with s do for i := 1 to length do write(tofile, letters[i]) 
end; (* writestring *)
(* end module interact.writestring version = 'prgmod 3.97  85 may 5 tds'; *)
  
(* begin module trigger.proc *) 
(* this module allows one to scan a series of characters, as from 
an array or a file, and to "trigger" or detect a simple string  
in the series.  the advantage of the trigger is that several triggers 
can "observe" a stream of characters at once, each looking for a
different thing.  
some other modules required: interact.const, interact.type *) 
  
procedure resettrigger(var t: trigger); 
(* reset the trigger to ground state *) 
begin (* resettrigger *)
      with t do begin 
         state := 0;
         skip := false; 
         found := false 
      end 
end; (* resettrigger *) 
  
procedure testfortrigger(ch: char; var t: trigger); 
(* look at the character ch.
   if it is part of the trigger (at the current trigger state), 
       then the trigger state goes higher.  
   if it is not part of the trigger then the trigger state is reset,  
      skip is true and one should skip onward to find the trigger.  
   if the trigger is found, found is true. *) 
begin (* testfortrigger *)
      with t do begin 
         state := succ(state);  
(*         if debugging then begin  
            writestring(list,seek); 
            writeln(list,'testfortrigger seek.letters[',state:1,']:', 
                           seek.letters[state],' ch:',ch);
         end;*) 
         if seek.letters[state] = ch  
         then begin 
            skip := false;
            if state = seek.length then found := true 
                                   else found := false  
         end
         else begin (* reset trigger *) 
            state := 0; 
            skip := true; 
            found := false
         end
      end 
end; (* testfortrigger *) 
(* end module trigger.proc version = 'prgmod 3.97  85 may 5 tds'; *)

(* begin module filler.fillstring *)
procedure fillstring(var s: string; a: filler); 
(* this procedure makes it reasonably easy to fill the string s with
characters.  one calls the procedure as: *) 
(*                           1         2         3         4         5 *) 
(*                  12345678901234567890123456789012345678901234567890 *) 
(*   fillstring(s, 'this-is-the-string                                ');   
the two comments make it easy to line the characters up. also, for this 
example, it was assumed that the length of filler as defined by the 
constant fillermax was 50. *) 
var 
      length: integer; (* of the string without trailing blanks *)  
      index: integer; (* of s *)  
begin (* fillstring *)  
      clearstring(s); 
      length := fillermax;  
      while (length > 1) and (a[length] = ' ') do length := pred(length); 
      if (length = 1) and (a[length] = ' ') then begin  
         writeln(output, 'fillstring: the string is empty');
         halt 
      end;  
  
      for index := 1 to length do s.letters[index] := a[index]; 
      s.length := length; 
      s.current := 1
end; (* fillstring *) 
(* end module filler.fillstring version = 'prgmod 3.97  85 may 5 tds'; *)

(* begin module filler.filltrigger *) 
procedure filltrigger(var t: trigger; 
                      a: filler); 
(* fill the trigger t *)  
begin (* filltrigger *) 
         fillstring(t.seek,a) 
end; (* fillstring *) 
(* end module filler.filltrigger version = 'prgmod 3.97  85 may 5 tds'; *)

(* begin module copyaline *)
procedure copyaline(var fin, fout: text); 
(* copy a line from file fin to file fout *)
begin (* copyaline *) 
      while not eoln(fin) do begin  
         fout^ := fin^;   
         put(fout);   
         get(fin) 
      end;  
      readln(fin);  
      writeln(fout);  
end; (* copyaline *)
(* end module copyaline version = 'prgmod 3.97  85 may 5 tds'; *)

(* ********************************************************************** *)
  
(* begin module skipblanks *) 
procedure skipblanks(var thefile: text);  
(* skip over blanks until a non-blank, or end of line, is found *)
begin 
      while (thefile^ = ' ') and not eoln(thefile) do get(thefile); 
end;  
  
procedure skipnonblanks(var thefile: text); 
(* skip over nonblanks until a blank, or end of line, is found *) 
begin 
      while (thefile^ <> ' ') and not eoln(thefile) do get(thefile);  
end;  
(* end module skipblanks version = 'prgmod 3.97  85 may 5 tds'; *)
(* ********************************************************************** *)

(* begin module genbank.objonline *)
procedure objonline(var object: trigger; var line: string);
(* is the object on the line?  object.found will be true if it is,
and the variable line.current will
point to the last character of the object *)
begin
      line.current := 0;
      resettrigger(object);
      while (line.current <= line.length) and
            (not object.found) do begin
         line.current := line.current +1;
         testfortrigger(line.letters[line.current],object); 
      end
(*;writeln(output,'objonline: ');writestring(output,line);writeln(output);
write(output,'object:');writestring(output,object.seek);writeln(output);
if object.found then writeln(output,'object was found')
else writeln(output,'no object found'); *)
end;
(* end module genbank.objonline *)

(* begin module genbank.find *)
procedure find(var thefile: text; object: trigger; var line: string);
(* find the object defined by the trigger in thefile.
return the entire line that the object was found on in the variable 'line'.
the current location in the string is the end of the object.
if no object was found, thefile will be at end of file. *)
var
      ch: char; (* a character in thefile *)
      gotten: boolean; (* for getstring: true if not eof *)
begin
      resettrigger(object);  
      while (not object.found) and (not eof(thefile)) do begin
         getstring(thefile,line,gotten);
         if gotten then objonline(object,line)
      end
end;
(* end module genbank.find *)

(* begin module genbank.findat *)
procedure findat(var thefile: text; var object: trigger;
                 spot: integer; var idline: string);
(* find the object at 'spot' on the line in thefile.  return the line
found in idline *)
var
      done: boolean; (* when have found valid entry start *)
begin
      done := false;
      while (not done) and (not eof(thefile)) do begin
         find(thefile,object,idline);
         (* make sure that the trigger is at the beginning of the line *)
         if idline.current=object.seek.length+spot-1 then done := true
      end
end;
(* end module genbank.findat *)

(* begin module genbank.findentry *)
procedure findentry(var thefile: text; var idline: string);
(* locate the beginning of the next genbank entry sequence in thefile *)
var
      entrystart: trigger; (* a trigger that locates the start of a sequence *)
begin
(*                                       1         2 *) 
(*                              12345678901234567890 *)
filltrigger(entrystart        ,'LOCUS               ');   
      findat(thefile,entrystart,1,idline);
end;
(* end module genbank.findentry *)

(* begin module genbank.endfeature *)
function endfeature(var l: string): boolean;
(* does the string contain text that indicates that a feature section is
finished? *)
var
      sites, bc, ori: trigger; (* triggers for end of feature section *)
begin
(*                                       1         2 *) 
(*                              12345678901234567890 *)
filltrigger(sites             ,'SITES               ');   
filltrigger(bc                ,'BASE COUNT          ');   
filltrigger(ori               ,'ORIGIN              ');   
      objonline(sites,l);
      objonline(bc,l);
      objonline(ori,l);
      endfeature := sites.found or bc.found or ori.found
end;
(* end module genbank.endfeature *)

(* begin module genbank.findfeature *)
function findfeature(var thefile: text; var l: string): boolean;
(* locate the beginning of the next feature in thefile,
and return true if one was found before end of feature signals were *)
var
      gotten: boolean; (* when not end of the file *)
      feature: trigger; (* a trigger that locates the start of a feature
            section of a genbank entry *)
begin
(*                                       1         2 *) 
(*                              12345678901234567890 *)
filltrigger(feature           ,'FEATURE             ');   
      repeat
         getstring(thefile,l,gotten);
(* writeln(output,'findfeature: ');writestring(output,l);writeln(output);*)
         if gotten then objonline(feature,l)
      until endfeature(l) or feature.found or eof(thefile);
      findfeature := feature.found and not eof(thefile)
end;
(* end module genbank.findfeature *)

(* begin module genbank.findsequence *)
procedure findsequence(var thefile: text; var idline: string);
(* locate the beginning of the next genbank entry sequence in thefile,
return the line before the start of the sequence in idline *)
var
      ch: char; (* a character in thefile *)
      seqstart: trigger; (* a trigger that locates the start of a sequence *)
begin
(*                                       1         2 *) 
(*                              12345678901234567890 *)
filltrigger(seqstart          ,'ORIGIN              ');   
      findat(thefile,seqstart,1,idline);
end;
(* end module genbank.findsequence *)

(* begin module genbank.getint *)
function getint(var l: string): integer;
(* get an integer from the current position in idline.
update the current position *)
var 
      done: boolean; (* when end of number is found *)
      increment: integer;  (* amount to increase the number by for each
         character in the string *) 
      sign: integer; (* the sign of the number *)
      sum: integer; (* running sum of the number *) 
begin (* figureinteger *) 
      sum:=0; (* start sum at zero *) 

      while (l.current <= l.length) and (l.letters[l.current]=' ')
         do l.current := l.current + 1; (* skip preceeding blanks *)

      if l.letters[l.current]='-' then begin
         sign := -1;
         l.current:=l.current+1
      end
      else if l.letters[l.current]='+' then begin
         sign := +1;
         l.current:=l.current+1
      end
      else sign := +1;

      while (l.current <= l.length) and (l.letters[l.current]
            in ['0','1','2','3','4','5','6','7','8','9']) do begin
         case l.letters[l.current] of
            '0': increment:=0;  
            '1': increment:=1;  
            '2': increment:=2;  
            '3': increment:=3;  
            '4': increment:=4;  
            '5': increment:=5;  
            '6': increment:=6;  
            '7': increment:=7;  
            '8': increment:=8;  
            '9': increment:=9 
         end; 
         sum := sum*10 + increment;
         l.current:=l.current+1
      end;
      getint := sign * sum
end;
(* end module genbank.getint *)

(* begin module genbank.getbase *)
procedure getbase(var thefile: text; var base: char; var gotten: boolean);
(* get a base from the genbank file, thefile.
if a base is found, then gotten is true.
if the end of the entry is found, gotten is false.
   mechanism:  getbase just reads characters.  blanks are skipped at all
points.  numbers are ignored, and other characters are returned.
the end of entry symbol (//) causes the base to be '/' and gotten to
be false when found.

note: getbase assumes that either findsequence or getbase was called
previously.  the procedure also requires the skipblanks module. *)
var
      done: boolean; (* true when a character was found or the end of the
         entry was detected *)
begin
      done := false;
      while not done do begin
         skipblanks(thefile);
         if not eoln(thefile) then begin
            read(thefile,base);
(*   writeln(output,'base=',base); *)
            if base = '/' then begin
               read(thefile,base);
               if base <> '/' then begin
                  writeln(output,'genbank entry ends with only one /');
                  halt
               end;
               done := true;
               gotten := false
            end
            else if not (base in ['0','1','2','3','4','5','6','7','8','9'])
                 then begin
                    done := true;
                    gotten := true
                 end
         end
         else readln(thefile)
      end
end;
(* end module genbank.getbase *)

(* begin module themain *)
procedure themain(var entries: text); 
(* the main procedure of the program *) 
var
      base: char; (* a base from the entry *)
      count: integer; (* number of bases in the entry so far *)
      gotten: boolean; (* true when a base was found in the from file *)
      line: string; (* a line from genbank *)
      number: integer; (* count of number of entries processed *)
procedure give;
(* give the line if it was found *)
begin
      if not eof(entries) then begin
         writestring(output,line);
         writeln(output);
      end
end;
begin
      writeln(output,'genmod ',version:4:2);

      reset(entries);
      number := 0;
      while not eof(entries) do begin
         findentry(entries,line); give;
         if findfeature(entries, line) then begin
            writeln(output,'the features are: ');
            repeat
               getstring(entries,line,gotten);
               if gotten then if not endfeature(line) then give;
            until endfeature(line) or not gotten;
            writeln(output,'end of features')
         end;
         findsequence(entries,line); give;
        
         if not eof(entries) then begin
            number := number + 1;
            writeln(output,'entry number ',number:1);
            count := 0;
            repeat
               getbase(entries,base,gotten);
               if gotten then begin
                  write(output,base);
                  count := count + 1;
                  if count >= 60 then begin
                     writeln(output);
                     count := 0
                  end
               end
            until not gotten;
            writeln(output)
         end
      end;
end;  
(* end module themain *)
 
begin
      themain(entries);
1: end.
