program htmlink(input, htmlinkp, list, output);
(* htmlink: insert html links to program references in a file

  Thomas D. Schneider, Ph.D.
  toms@alum.mit.edu
  https://alum.mit.edu/www/toms 

 *)

label 1; (* end of program *)

const
(* begin module version *)
   version = 1.62; (* of htmlink.p 2016 Oct 15
2016 Oct 14: 1.62: clean up
2016 Oct 14: 1.61: debug
2016 Oct 14: 1.60: upgrade modules
2016 Oct 14: 1.59: allow width defined for images
2016 Jan 22: 1.58: program.p links are not working
2016 Jan 22: 1.57: remove ftp links
2016 Jan 22: 1.56: handle https (building with p2c/tc)
2011 Aug 24: 1.55: align alt tags
2011 Apr 05: 1.54: fix alt tags
2011 Apr 05: 1.53: add alt tags to images
2004 Sep 15: 1.52: document upgrade
2004 Jul 29: 1.48: update url for gpc
2004 Jul 29: 1.47: use strong
1.46; 2000 Oct 24: implement ftp://
                   New as of 2000 Oct 24: names that begin with ftp://
                   are presented as straight html links.
1.40; 2000 Jul 16: implement links for images: .gif and .jpg
1.39; 2000 Jul 15: "see also" only recognized at start of line!!
1.36; 2000 Jun 20: make {} comment marks go away on the html page.
1.35; 2000 Jan 10: fix bug in final pointer at page bottom
1.28; 2000 Jan  3: upgrade to recognize http links, clean up and do {}.
                   New as of 2000 Jan 3: names that begin with http://
                   are presented as straight html links.
1.27; 1998 Mar 30: previous changes
origin 1995 March 10 *)
(* end module version *)

(* begin module describe.htmlink *)
(*
name

   htmlink: insert html links to program references in a file

synopsis

   htmlink(input: in, htmlinkp: in, list: out, output: out)

files

   input:  a delila module or other file containing references
      to programs in the form *.p and references to other files between
      the key words "see also" and "author".

      Comments: the material contained within curlie brackets, "{" and "}",
      will not be processed into hyper links.  To create one of these
      characters, put a backslash "\" in front of it.

      The key words "see also" and "author" MUST be at the beginning of the
      line.  This allows one to say "see also" in the middle of other
      descriptions.  This is not a harsh restriction, since everything is
      indented as a standard anyway!

   htmlinkp:  A parameter file that defines:

      first line:  the ARCHIVE directory where files other
         than programs should be linked to.
      second line: the MANUAL directory where programs are linked to.
      third line:  font size for the output.

   list: a list of the files being linked to.

   output: the same file with the *.p converted to references to *.html, and
      the other references converted to hypertext links to
      ARCHIVE/reference.  To prevent weird links from being made, this only
      occurs between the lines 'see also' and 'author', when these key words
      are at the start of the line.

      Names that begin with http:// or https:// are presented as
      straight html links.

      names that begin with ftp:// are presented as straight html
      links.

description

   This program allows one to convert the Delila manual pages to hypertext
   linked objects.

   The program copies the input to the output, and only modifies the region
   between key words 'see also' and 'author' when these are left justified on
   the line.  This defines the standard place that Delila documentation
   points to other files.

   Pascal programs ending in ".p" are made into the form:

   [A HREF ="MANUAL/doodle.html"]doodle.p[/A]

   (where the program is 'doodle.p and I have replaced "less than" with "["
   and "greater than" with "]" to prevent the browser from interpreting
   this as a link.) so that they will point to other programs
   in the same directory, while other files are made into the form:
   MANUAL is a string read in from the htmlinkp parameter file.

   [A HREF ="ARCHIVE/xyplop"]xyplop[/A]

   where ARCHIVE is a string read in from the htmlinkp parameter file and the
   file is "xyplop".

   Alt tags are added to the images constructed from the image name.

   The program is used by the shell script "dth" (delila to html).

examples

   Below is the material in the "see also" section.  After processing, you
   will see that the text looks the same except that things outside the
   curlie braces are hyperlinked.

{\{ left curlie}
{ \{ left curlie}
{ { left curlie}
{\} right curlie}
{ \} right curlie}
{ \} right curlie}
{ \ escape}
{ \\ escape}
{ \\\ escape}

   {Some programs that use this processing:}
   alist.p
   delila.p

   {A general http hyperlink is demonstrated by google at:}
   http://google.com

   {A general https hyperlink is demonstrated by my home page which is at:}
   https://alum.mit.edu/www/toms/

   {The mechanism even allows images!}
   https://alum.mit.edu/www/toms/images/tinygumball.gif

   {The script for running the Gnu Pascal Compiler:}
   https://alum.mit.edu/www/toms/ftp/gpcc
   {(The GNU Pascal Compiler is at}
   http://www.gnu-pascal.de/gpc/h-index.html{)}

documentation

see also

   {Example parameter file:} htmlinkp

   {A program that extracts the manual page of Delila programs:} makman.p

   {A general script for storing Delila programs in an archive:} dth

   {Here is the processing of the examples above.}
{\{ left curlie}
{ \{ left curlie}
{ { left curlie}
{\} right curlie}
{ \} right curlie}
{ \} right curlie}
{ \ escape}
{ \\ escape}
{ \\\ escape}

   {Some programs that use this processing:}
   alist.p
   delila.p

   {A general http hyperlink is demonstrated by google at:}
   http://google.com

   {A general https hyperlink is demonstrated by my home page which is at:}
   https://alum.mit.edu/www/toms/

   {The mechanism even allows images!}
   https://alum.mit.edu/www/toms/images/tinygumball.gif

   {The script for running the Gnu Pascal Compiler:}
   https://alum.mit.edu/www/toms/ftp/gpcc
   {(The GNU Pascal Compiler is at}
   http://www.gnu-pascal.de/gpc/h-index.html{)}

author
   Thomas Dana Schneider

bugs

   The triggers "see also" and "author" and their positions on the
   line should be generalized to allow the program to process any
   text.  This would not be hard since it would only require new
   parameters being read in.

   The 'see also' needs a stable example of an ftp link.

technical notes

  Note:  this program surrounds the file with [PRE] and [/PRE].

*)
(* end module describe.htmlink *)

(* more constants *)
   maxline = 100; (* longest line allowed *)
   fontsize = 2; (* size of fonts *)

(* begin module interact.const *)
(* begin module string.const *)
      maxstring = 2000; (* the maximum string *)
(* end module string.const version = 5.54; (@ of prgmod.p 2016 Jul 20 *)
(* end module interact.const version = 5.54; (@ of prgmod.p 2016 Jul 20 *)

(* begin module filler.const *)
      fillermax = 50; (* the size of the filler array for a string *)
(* end module filler.const version = 5.54; (@ of prgmod.p 2016 Jul 20 *)

type

(* begin module interact.type *)
(* begin module string.type *)
   stringptr = ^string; (* pointer to a string *)
   string = record (* a string of characters *)
      letters: array[1..maxstring] of char; (* the letters in the string *)
      length: integer; (* the number of characters in the string *)
      current: integer; (* the letter we are working on *)
      previous: stringptr; (* the previous string in a series *)
      next: stringptr; (* the next string in a series *)
   end;
(* end module string.type version = 5.54; (@ of prgmod.p 2016 Jul 20 *)
(* end module interact.type version = 5.54; (@ of prgmod.p 2016 Jul 20 *)

(* begin module trigger.type *)
      trigger = record (* an object to be searched for *)
         seek: string; (* the characters looked for *)
         state: integer; (* how close to triggering we are *)
         skip: boolean; (* trigger not found- skip the line *)
         found: boolean (* the trigger was found *)
      end;
(* end module trigger.type version = 5.54; (@ of prgmod.p 2016 Jul 20 *)

(* begin module filler.type *)
      (* the following is an array used to fill a string.
      it is convenient to have it much shorter than the maxstring, so that
      it is easy to fill the string using procedure fillstring.
      the user must declare the value of constant fillermax. *)
      filler = packed array[1..fillermax] of char;
(* end module filler.type version = 5.54; (@ of prgmod.p 2016 Jul 20 *)

var
(* begin module const.htmlink *)
   htmlinkp: text; (* parameters to control the program *)
   list: text; (* list of files linked to *)
(* end module const.htmlink *)

(* begin module halt *)
procedure halt;
(* stop the program.  the procedure performs a goto to the end of the
   program.  you must have a label:
      label 1;
   declared, and also the end of the program must have this label:
      1: end.
   examples are in the module libraries.
   this is the only goto in the delila system. *)
begin
      writeln(output,' program halt.');
      goto 1
end;
(* end module halt version = 5.54; (@ of prgmod.p 2016 Jul 20 *)

(* begin module interact.clearstring *)
(* begin module clearstring *)
(* These modules clear strings in various ways *)
(* ---- *)
procedure emptystring(var ribbon: string);
(* empty the contents of the string but do NOT
remove the pointer.  This is useful for clearing
one string within a linked list of them. *)
var   index: integer; (* to the ribbon *)
begin (* clearstring *)
   with ribbon do begin
      for index := 1 to maxstring do letters[index] := ' ';
      length := 0;
      current := 0;
   end
end; (* emptystring *)
(* ---- *)
procedure clearstring(var ribbon: string);
(* empty the string and remove the pointer.
As of 2010 Dec 04 there is a previous pointer that
allows proper removal of the linked list using disposestring.
This clearstring routine is for the initial clearing
of the variables. *)
begin (* clearstring *)
   with ribbon do begin
      emptystring(ribbon);
      previous := nil;
      next := nil;
   end
end; (* clearstring *)
(* ---- *)
procedure disposestring(var ribbon: stringptr);
(* remove the linked list from the string in revere order *)
var
   t: stringptr; (* a link in the string *)
begin
   t := ribbon;
   (* find the end *)
   while t^.next <> nil do t := t^.next;
   (* move backwards and dispose links *)
   while t^.previous <> nil do begin
      t := t^.previous;
      dispose(t^.next);
   end;
   dispose(t);
end; (* disposestring *)
(* ---- *)
procedure initializestring(var ribbon: string);
(* start the string with a nil pointer.   This routine
should be called before doing linked list work.  This allows
the standard string routines to clear the string without
killing the pointer.  This is now deprecated, do not use it
since clearstring still clears the next pointer. *)
begin (* initializestring *)
   writeln(output,'remove initializestring routine!');
   writeln(output,'replace it with clearstring routine!');
   halt; (* to force deprecation *)
   clearstring(ribbon);
   ribbon.next := nil;
end; (* initializestring *)
(* end module clearstring version = 5.54; (@ of prgmod.p 2016 Jul 20 *)
(* end module interact.clearstring version = 5.54; (@ of prgmod.p 2016 Jul 20 *)

(* begin module filler.fillstring *)
procedure fillstring(var s: string; a: filler);
(* this procedure makes it reasonably easy to fill the string s with
characters.  one calls the procedure as: *)
(*                           1         2         3         4         5 *)
(*                  12345678901234567890123456789012345678901234567890 *)
(*   fillstring(s, 'this-is-the-string                                ');
the two comments make it easy to line the characters up. also, for this
example, it was assumed that the length of filler as defined by the
constant fillermax was 50. *)
var
      length: integer; (* of the string without trailing blanks *)
      index: integer; (* of s *)
begin (* fillstring *)
      clearstring(s);
      length := fillermax;
      while (length > 1) and (a[length] = ' ') do length := pred(length);
      if (length = 1) and (a[length] = ' ') then begin
         writeln(output, 'fillstring: the string is empty');
         halt
      end;

      for index := 1 to length do s.letters[index] := a[index];
      s.length := length;
      s.current := 1
end; (* fillstring *)
(* end module filler.fillstring version = 5.54; (@ of prgmod.p 2016 Jul 20 *)

(* begin module filler.filltrigger *)
procedure filltrigger(var t: trigger;
                      a: filler);
(* fill the trigger t *)
begin (* filltrigger *)
         fillstring(t.seek,a)
end; (* filltrigger *)
(* end module filler.filltrigger version = 5.54; (@ of prgmod.p 2016 Jul 20 *)
(* begin module trigger.proc *)
(* this module allows one to scan a series of characters, as from
an array or a file, and to "trigger" or detect a simple string
in the series.  the advantage of the trigger is that several triggers
can "observe" a stream of characters at once, each looking for a
different thing.
some other modules required: interact.const, interact.type *)

procedure resettrigger(var t: trigger);
(* reset the trigger to ground state *)
begin (* resettrigger *)
      with t do begin
         state := 0;
         skip := false;
         found := false
      end
end; (* resettrigger *)

procedure testfortrigger(ch: char; var t: trigger);
(* look at the character ch.
   if it is part of the trigger (at the current trigger state),
       then the trigger state goes higher.
   if it is not part of the trigger then the trigger state is reset,
      skip is true and one should skip onward to find the trigger.
   if the trigger is found, found is true.
1996 Sep 12: Bug found!  In the case of a trigger "ab", the program
used to miss it for situations like "aab".  This was because at the
first a it would step up.  Then it would see the second a and recognize
that was not part of ab.  It would fail to realize that it could be
the start of a new one.  The code now accounts for that possibility. *)
begin (* testfortrigger *)
      with t do begin
         state := succ(state);
{
writestring(list,seek);
writeln(list,'testfortrigger seek.letters[',state:1,']:',
seek.letters[state],' ch:',ch);
}
         if seek.letters[state] = ch
         then begin
            skip := false;
            if state = seek.length then found := true
                                   else found := false
         end
         else begin
            (* it failed.  But wait!  It could be the beginning
            of a NEW trigger string! *)
            if seek.letters[1] = ch then begin
               state := 1;
               skip := false;
               found := false
            end
            else begin (* reset trigger *)
               state := 0;
               skip := true;
               found := false
            end
         end
      end
end; (* testfortrigger *)
(* end module trigger.proc version = 5.54; (@ of prgmod.p 2016 Jul 20 *)

(* begin module interact.getstring *)
procedure getstring(var afile: text; var buffer: string;
                    var gotten: boolean);
(* get a line (as a string) from a file not using string calls.  this lets
one obtain lines from a file without interactive prompts *)
var   index: integer; (* of buffer *)
begin (* getstring *)
      clearstring(buffer);
      if eof(afile)
      then gotten := false
      else begin
         index := 0;
         while (not eoln(afile)) and (index < maxstring) do begin
            index := succ(index);
            read(afile, buffer.letters[index])
         end;
         if not eoln(afile) then begin
            writeln(output, ' getstring: a line exceeds maximum string size (',
                            maxstring:1,')');
            halt
         end;
         buffer.length := index;
         buffer.current := 1;
         readln(afile);
         gotten := true
      end
end; (* getstring *)
(* end module interact.getstring version = 5.54; (@ of prgmod.p 2016 Jul 20 *)

(* begin module interact.writestring *)
(* begin module writestring *)
procedure writestring(var tofile: text; var s: string);
(* write the string s to file tofile, no writeln *)
var   i: integer; (* index to s *)
begin (* writestring *)
      with s do for i := 1 to length do write(tofile, letters[i])
end; (* writestring *)
(* end module writestring version = 5.54; (@ of prgmod.p 2016 Jul 20 *)
(* end module interact.writestring version = 5.54; (@ of prgmod.p 2016 Jul 20 *)

(* begin module htmlink.writealine *)
procedure writealine(var fout: text; aline: string; start, stop: integer;
                     escape: char);
(* write the aline string to fout from start to stop.  Do not show
escape symbols. *)
var
  c: char; (* a character in aline *)
  i: integer; (* index to aline *)
  p: char; (* the character before c *)
begin
  c := ' ';
  for i := start to stop do begin
     p := c;
     c := aline.letters[i];
     if p = escape
     then write(fout,c)
     else if (c <> escape) and (p <> escape)
          then write(fout,c);
{
     if (c <> escape) and (p <> escape)
     then write(fout,c);
}
  end;
end;
(* end module htmlink.writealine *)

procedure copyindentation(aline: string);
(* copy the indentation of the aline string to output *)
var i: integer;
begin
   (* copy indentation *)
   i := 1;
   while (aline.letters[i] = ' ') or
         (aline.letters[i] = chr(9)){ie tab}
   do begin
      write(output, aline.letters[i]);
      i := succ(i);
   end;
end;

(* begin module copystring *)
procedure copystring(a: string; var b: string);
(* copy string a to b.  The length, characters and
where the string points to are all copied. *)
var l: integer; (* index to the string *)
begin
   b.length := a.length;
   for l := 1 to a.length do b.letters[l] := a.letters[l];
   b.current := a.current;
   b.next := a.next;
end;
(* end module copystring version = 5.54; (@ of prgmod.p 2016 Jul 20 *)

(* begin module htmlink.themain *)
procedure themain(var fin, htmlinkp, list: text);
(* the main procedure of the program *)
const
   escape = '\'; (* the character that prevents processing of { or } *)
var

   ARCHIVE: string; (* the location of other files *)
   MANUAL: string; (* the location of Delila manual pages *)
   aline: string; (* a line of text *)
   beforedot: integer; (* location just before '.' in aline *)
   c: char; (* a character being processed *)
   comment: boolean; (* true when inside a comment *)
   insidecomment: boolean; (* previous stuff put us inside a comment,
      so if we see a \{ or \} these should be SHOWN. *)
   cprevious: char; (* the character before c *)
   done: boolean; (* done parsing a link? *)
   l: integer; (* current position in the buffer, and its length *)
   fontsize: integer; (* html font size *)
   gotten: boolean; (* was htmlinkp readable? *)
   follow: integer; (* index to b that follows behind i *)
   i: integer; (* index to b *)
   image: boolean; (* true when the link should be an image *)
   linkprocess: boolean; (* true only when we want to process a link *)
   seealsocount: integer; (* count of 'see also's to prevent more than
      the first in the file from being used *)
   slashdone: boolean; (* done looking for a slash? *)
   slash: integer; (* location of slash in aline *)
   potentiate: boolean; (* becomes true when we want to process a link.
      We don't want to process on the same line that "see also" is because
      then "see also" becomes part of the html rather than the copied
      material.  So we hold off (by using this variable) one line
      before turning linkproces on. *)
   pureurl: boolean; (* is this a pure url and not an archive link? *)

   seealso: trigger; (* trigger for "see also" *)
   author: trigger; (* trigger for "author" *)
   http: trigger; (* trigger for "http" *)
   https: trigger; (* trigger for "https" *)
   ftp: trigger; (* trigger for "ftp" *)
   gif: trigger; (* trigger for ".gif" *)
   jpg: trigger; (* trigger for ".jpg" *)

   width: trigger; (* trigger for "width" *)
   imagecontrol: string; (* text for controlling an image *)
   suppress: boolean; (* suppress the image control line *)

   numbers: set of '0'..'9'; (* the digits *)

begin
   (* set up the triggers *)
(*                             1         2         3         4         5 *)
(*                    12345678901234567890123456789012345678901234567890 *)
filltrigger(seealso, 'see also                                          ');
filltrigger(author , 'author                                            ');
filltrigger(http   , 'http://                                           ');
filltrigger(https  , 'https://                                          ');
filltrigger(ftp    , 'ftp://                                            ');
filltrigger(gif    , '.gif                                              ');
filltrigger(jpg    , '.jpg                                              ');
filltrigger(width  , 'width                                             ');

   reset(htmlinkp);
   if eof(htmlinkp) then begin
      writeln(output,'htmlinkp is empty');
      halt
   end;

   getstring(htmlinkp,ARCHIVE,gotten);
   if not gotten then begin
      writeln(output,'Could not get ARCHIVE pointer');
      halt
   end;

   numbers:=['0','1','2','3','4','5','6','7','8','9'];
   if htmlinkp^ in numbers then begin
      write(output,'BAD PARAMETERS:  ');
      writeln(output,'htmlinkp expects 3 parameters now; see the manual');
      halt;
   end;

   getstring(htmlinkp,MANUAL,gotten); (* NEW as of 2016 Jan 22 *)
   if not gotten then begin
      writeln(output,'Could not get MANUAL pointer in htmlinkp');
      halt
   end;

   if eof(htmlinkp) then begin
      writeln(output,'missing fontsize parameter in htmlinkp');
      halt
   end;

   readln(htmlinkp, fontsize);

   rewrite(list);
   writeln(output,'<!-- htmlink ',version:4:2 ,' -->');
   writeln(output,'<font size = ',fontsize:1,'>');
   writeln(output,'<strong>');
   writeln(output,'<PRE>');

   clearstring(aline);
   linkprocess := false;
   potentiate := false;
   comment := false;
   seealsocount := 0;
   suppress := false;
   clearstring(imagecontrol);
   while not eof(fin) do begin

      (* read in a line of text *)
      getstring(fin,aline,gotten);
      if not gotten then begin
         writeln(output,'htmlink: program error?');
         halt
      end;

      resettrigger(seealso);
      resettrigger(author);
      for l := 1 to aline.length do begin
         c := aline.letters[l];

         testfortrigger(c, seealso);
         if l = 8 then begin (* "see also" is 8 characters long *)
            if seealso.found then begin
               seealsocount := succ(seealsocount);
               if seealsocount = 1 then potentiate := true;
            end;
         end;

         testfortrigger(c, author);
         if l = 6 then begin (* "author" is 8 characters long *)
            if author.found then begin
               linkprocess := false;
               potentiate := false;
            end;
         end;
      end;

      if linkprocess then begin (* process the link section *)

         (* parse out the named objects on one line *)
         follow := 1;

         pureurl := false;
         c := ' ';
suppress := false;
         while follow <= aline.length do begin

{
writeln(output,'l=',follow:1);
}
            cprevious := c;
            c := aline.letters[follow];
            insidecomment := (comment = true);
            if c = '{' then comment := true;
            if (cprevious <> escape) and (c = '}') then comment := false;
            if c = ' ' then write(output,' ');
            if c = ' ' then write(list,' ');
            if insidecomment and (cprevious = escape) then begin
{
            if cprevious = escape then begin
}
               if c = '{' then write(output,'{');
               if c = '{' then write(list,'{');
               if c = '}' then write(output,'}');
               if c = '}' then write(list,'}');
            end;

            if (c <> ' ') and (c <> ',')
            and (c <> '{') and (c <> '}')
            then begin
               (* parse *)
               resettrigger(http);
               resettrigger(https);
               resettrigger(ftp);
               resettrigger(gif);
               resettrigger(jpg);
               resettrigger(width);
               image := false;
               l := follow;
{
writeln(output); write(output,'parsed string: "');
}
               done := false;
               while not done do begin
                  testfortrigger(aline.letters[l], http);
                  if http.found then pureurl := true;

                  testfortrigger(aline.letters[l], https);
                  if https.found then pureurl := true;

                  testfortrigger(aline.letters[l], ftp);
                  if ftp.found then pureurl := true;

                  testfortrigger(aline.letters[l], gif);
                  if gif.found then image := true;

                  testfortrigger(aline.letters[l], jpg);
                  if jpg.found then image := true;

                  testfortrigger(aline.letters[l], width);
                  if width.found then begin
                     copystring(aline, imagecontrol);
                     suppress := true;
                  end;

{
write(output,aline.letters[l]);
}

                  if (l = aline.length)
                  then done := true
                  else begin
                     l := succ(l);
                     if (aline.letters[l] = ' ') or
                        (aline.letters[l] = ',') or
                        (aline.letters[l] = '}') or
                        (aline.letters[l] = '{')
                     then begin
                         done := true;
                         l := pred(l); (* don't include that *)
                     end
                  end
               end;
               if not suppress then begin {zzz}
                  if comment then begin
                     writealine(output, aline, follow, l, escape);
                     writealine(list,   aline, follow, l, escape);
                  end
                  else begin
                     (* if we just found *.p, write it out as HTML *)
                     if (aline.letters[l-1] = '.') and (aline.letters[l] = 'p')
                     then begin
                        write(output,'<A HREF ="');
                        writestring(output,MANUAL);
                        for i := follow to l-1 do write(output,aline.letters[i]);
                        write(output,'html">'); (* replaces .p with .html *)
                        for i := follow to l do write(output,aline.letters[i]);
                        write(output,'</A>');
                        for i := follow to l do write(list,aline.letters[i]);
                     end

                     else begin

                        if image then begin
                           writeln(output,'<img');
{zzz}
                           if imagecontrol.length > 0 then begin
                              writestring(output,imagecontrol);
                              clearstring(imagecontrol); (* clear for next use *)
                              writeln(output);
                              suppress := false;
                           end;
                           copyindentation(aline);
                           write(output,'src="');
                        end
                        else write(output,'<a href="');

                        if not pureurl then begin
                           writestring(output,ARCHIVE);
                        end;
                        for i := follow to l
                        do write(output,aline.letters[i]);
                        write(output,'"');
                        if image then begin (* add alt tag for images *)
                           writeln(output);
                           copyindentation(aline);
                           write(output,'alt="');
                           (* locate the last slash in the url *)
                           slash := l;
                           beforedot := l;
                           slashdone := false;
                           while not slashdone do begin
                              if aline.letters[slash] = '/' then begin
                                 slashdone := true;
                              end
                              else slash := slash - 1;
                              (* find the last dot in the name *)
                              if aline.letters[slash] = '.' then begin
                                 if beforedot = l (* detect only once *)
                                 then beforedot := slash - 1
                              end;
                              if slash = follow then begin
                                 slashdone := true;
                                 (* make sure we include the first character! *)
                                 slash := slash - 1;
                              end;
                           end;
                           write(output,'image for ');
                           for i := slash+1 to beforedot
                           do write(output,aline.letters[i]);
                           write(output,'"');
                        end;
                        write(output,'>');

                        if not image then begin
                           (* complete the link by giving it again: *)
                           for i := follow to l
                           do write(output,aline.letters[i]);
                           write(output,'</A>');
                        end;

                        for i := follow to l
                        do write(list,aline.letters[i]);

                     end;
                  end;
               end; {zzz}

               follow := l; (* step parsing forward across the word *)
            end;
            if c = ',' then write(output,',');
            follow := succ(follow);

         end;

         writeln(output);
         writeln(list);
      end
      else begin (* just copy the line *)
         writestring(output,aline);
         writeln(output);
      end;

      (* this line had a "see also" so process the next line *)
      if potentiate then linkprocess := true;

   end;

   writeln(output,'</PRE>');
   writeln(output,'</strong>');
   writeln(output,'{<a href = "htmlink.html">created by htmlink ',
                  version:4:2 ,'</a>}');
end;
(* end module htmlink.themain *)

begin
   themain(input, htmlinkp, list);
1: end.
