program odti(od,time,odtime,output);
(* munch od and time plates together for xyplo
   tom schneider *)

label 1; (* the end of the program *)

const
(* begin module version *)
version = 1.03; (* of odti.p 1994 sep 5
origin 1986 oct 11 *)
(* end module version *)

(* begin module describe.odti *)
(*
name
   odti: munch od and time plates together for xyplo

synopsis
   odti(od: in, time: in, odtime: out, output: out)

files
   od: a file containing just an od plate from the tk program.
      blank wells are indicated by an '*'.
   time: a file containing times.
      blank wells are indicated by an '*'.
   odtime: the od and time values are spliced together,
      lines beginning with * are copied to output, then
      the time followed by each od are put on lines by themselves.
      this is the form that that the xyplo program can use for
      plotting.

description
   the od and time plates are fused together for plotting with xyplo.

auther
   tom schneider

see also
   xyplo.p and tk (written in basic)

bugs
   does not take full tk output.

*)
(* end module describe.odti *)

var
   od, time, odtime: text; (* input and output files *)

(* begin module halt *) 
procedure halt; 
(* stop the program.  the procedure performs a goto to the end of the   
   program.  you must have a label: 
      label 1;
   declared, and also the end of the program must have this label:  
      1: end. 
   examples are in the module libraries.
   this is the only goto in the delila system. *) 
begin 
      writeln(output,' program halt.'); 
      goto 1
end;  
(* end module halt *)

(* begin module copyaline *)
procedure copyaline(var fin, fout: text); 
(* copy a line from file fin to file fout *)
begin (* copyaline *) 
      while not eoln(fin) do begin  
         fout^ := fin^;   
         put(fout);   
         get(fin) 
      end;  
      readln(fin);  
      writeln(fout);  
end; (* copyaline *)
(* end module copyaline *)

(* begin module copylines *)
function copylines(var fin, fout: text; n: integer): integer; 
(* copy n lines of file fin to file fout. 
the actual number of lines copied is returned. *) 
var 
      index: integer; (* the current line number *) 
begin (* copylines *) 
      index := 0; 
      while (not eof(fin)) and (index < n) do begin 
         copyaline(fin, fout);  
         index := succ(index) 
      end;  
  
      copylines := index
end; (* copylines *)
(* end module copylines *)
  
procedure themain(var od, time, odtime: text);
(* the main procedure of the program.
read in od and time and fuse into odtime *)
var
   indexc: integer; (* index to the columns of both plates *)
   indexr: integer; (* index to the rows of both plates *)
   number: real; (* a number from one of the plates *)
begin
   writeln(output,'odti ',version:4:2);
   reset(od);
   reset(time);
   rewrite(odtime);

   if copylines(od,odtime,4)<>4 then begin
      writeln(output,'od file is not long enough');
      halt
   end;

   if copylines(time,odtime,6)<>6 then begin
      writeln(output,'time file is not long enough');
      halt
   end;

   readln(od);   (* skip the A to H line *)
   readln(time); (* skip the A to H line *)

   readln(od);   (* skip the blanking line *)
   readln(time); (* skip the blanking line *)

   for indexr := 2 to 12 do begin
      read(od,number);   (* skip the row number *)
      read(time,number); (* skip the row number *)

      for indexc := 1 to 8 do begin
         read(time, number);
         write(odtime, number:6:3);
         read(od, number);
         write(odtime, number:6:3);
         writeln(odtime)
      end
   end
end;

begin
   themain(od,time,odtime);
1: end. 
