program ordcut(input, ordcutp, output);
(* ordcut: cut out characters based on their ordinality

  Dr. Thomas D. Schneider
  toms@alum.mit.edu
  permanent email: toms@alum.mit.edu
  https://alum.mit.edu/www/toms/


 *)

label 1; (* end of program *)

const
(* begin module version *)
   version = 1.02; (* of ordcut.p 2003 Sep 3
2003 Sep  3, 1.02: make removed characters to be spaces
2003 Sep  3, 1.01: functional; touchup
2003 Sep  3, 1.00: origin
*)
updateversion = 1.00; (* defines lowest acceptable current parameter file *)
(* end module version *)

(* begin module describe.ordcut *)
(*
name
   ordcut: cut out characters based on their ordinality

synopsis
   ordcut(input: in, ordcutp: in, output: out, output: out)

files

   input:  a file

   output:  the input file with only characters within the range

   ordcutp:  parameters to control the program.  The file must contain the
      following parameters, one per line:

      parameterversion: The version number of the program.  This allows the
         user to be warned if an old parameter file is used.

      NOT USED

description

   The purpose of ordcut is to remove weird characters from a file.

   Now that it is written, I just realized that I can do the same
   kind of thing with:

   tr -cd [:print:]

   Yea, you would think.  But it turns ou tht that drops all
   the numbers on MacOSX!

   The removed characters are turned into spaces so that
   words can be extracted from the file.

examples

documentation

see also

   {Program that tells what characters are in a file:} wtch.p

author

   Thomas Dana Schneider

bugs

   The program does not read the parameter file yet.
   It simply gives printable characters, including
   tab, space, linefeed and carriage return.

technical notes

*)
(* end module describe.ordcut *)

var
   ordcutp{: (* file used by this program *)
   input, (* file used by this program *)
   output}: text; (* file used by this program *)

(* begin module halt *)
procedure halt;
(* stop the program.  the procedure performs a goto to the end of the
   program.  you must have a label:
      label 1;
   declared, and also the end of the program must have this label:
      1: end.
   examples are in the module libraries.
   this is the only goto in the delila system. *)
begin
      writeln(output,' program halt.');
      goto 1
end;
(* end module halt version = 'delmod 6.16 84 mar 12 tds/gds'; *)

(* begin module ordcut.themain *)
procedure themain(var {input,} ordcutp{, output}: text);
(* the main procedure of the program *)
var
{
   parameterversion: real; (* parameter version number *)
}
   c: char; (* a character *)
   lowerbound: integer; (* lowest ord of a character to accept  *)
   upperbound: integer; (* highest ord of a character to accept  *)
begin
{
   writeln(output,'ordcut ',version:4:2);
   reset(ordcutp);
   readln(ordcutp, parameterversion);
   if round(100*parameterversion) < round(100*updateversion) then begin
      writeln(output, 'You have an old parameter file!');
      halt
   end;
}

   lowerbound := -maxint;
   upperbound := +maxint;

   lowerbound := 32; (* space *)
   upperbound := 126; (* this is ~; 127 is del;
                         128 starts high order bits *)

{
   upperbound := 48; (* for testing *)
}

   while not eof (input) do begin
      if eoln(input)
      then begin
         readln(input);
         writeln(output);
      end
      else begin
         read(input,c);
         if ((ord(c) >= lowerbound) and
             (ord(c) <= upperbound))
             or
            (ord(c) = 9) (* tab *)
             or
            (ord(c) = 12) (* linefeed *)
             or
            (ord(c) = 13) (* Carriage Return *)
         then begin
            write(output,c);
         end
         else begin
            write(output,' ');
         end;
      end
   end;

end;
(* end module ordcut.themain *)

begin
   themain({input,} ordcutp{, output});
1: end.
