program pst(afile, bfile, output);
(* pst: paste afile to bfile and send to output

  Dr. Thomas D. Schneider
  toms@alum.mit.edu
  permanent email: toms@alum.mit.edu
  https://alum.mit.edu/www/toms/


 *)

label 1; (* end of program *)

const
(* begin module version *)
   version = 1.02; (* of pst.p 2004 Sep  8
2004 Sep  8, 1.02: remove duplicate const for gpc.
2004 Jul 22, 1.00: origin
*)
updateversion = 1.00; (* defines lowest acceptable current parameter file *)
(* end module version *)

(* begin module describe.pst *)
(*
name
   pst: paste afile to bfile and send to output

synopsis
   pst(afile: in, bfile: out, output: out)

files

   afile:  multiple line detailed description of the first file.

   bfile:  multiple line detailed description of the second file.

description

   The pst program is a replacement for Unix paste, which cannot
   handle lines longer than 511 characters!!

   The mechanism is to combine each line from afile with the
   corresponding line from bfile, with a tab separating them and put
   the result on output.

   There is no buffer in the program, so it can take arbitrarily long
   lines.

examples

afile:

AFILE
contains
this

bfile:

BFILE
has
that

The output is:

AFILE	BFILE
contains	has
this	that

documentation

see also

  {google for paste.c:}
  http://www.google.com/search?hl=en&ie=UTF-8&q=paste.c&btnG=Google+Search
  {gives:}
  http://docsrv.sco.com:507/en/man/html.C/paste.C.html
  {Diagnostics
  line too long
      Output lines are restricted to 511 characters.
  DUH!!!! Paste has this low limit!!}
  http://openbsd.mirrors.pair.com/src/usr.bin/paste/paste.c
  {coooooollllll!!
  http://openbsd.mirrors.pair.com/src/usr.bin/paste/
  {has Makefile, paste.1 and paste.c
  can't get it to compile - includes that I give it are not found!}

author

   Thomas Dana Schneider

bugs

   As pst is in standard Pascal, it does not read command lines.  It
   takes an afile and abfile and pastes them to output, Someday I
   should learn how to make non-standard calls to files on the
   argument line in gpc ... then it would look just like paste but
   work with arbitrarily long lines.

technical notes

*)
(* end module describe.pst *)

   separator = '	'; (* tab character *)

var
   afile, (* file used by this program *)
   bfile: text; (* file used by this program *)

(* begin module halt *)
procedure halt;
(* stop the program.  the procedure performs a goto to the end of the
   program.  you must have a label:
      label 1;
   declared, and also the end of the program must have this label:
      1: end.
   examples are in the module libraries.
   this is the only goto in the delila system. *)
begin
      writeln(output,' program halt.');
      goto 1
end;
(* end module halt version = 'delmod 6.16 84 mar 12 tds/gds'; *)

(* begin module pst.themain *)
procedure themain(var afile, bfile: text);
(* the main procedure of the program *)
var
   done: boolean; (* done pasting *)
   c: char; (* a character from either file, on its way to output *)
{
   parameterversion: real; (* parameter version number *)
}
begin
{
   writeln(output,'pst ',version:4:2);
   reset(pstp);
   readln(pstp, parameterversion);
   if round(100*parameterversion) < round(100*updateversion) then begin
      writeln(output, 'You have an old parameter file!');
      halt
   end;
}

   reset(afile);
   reset(bfile);

   done := false;
   while not done do begin
      if eof(afile) then done := true;
      if eof(bfile) then done := true;
      if not done then begin

         (* first copy a line of the afile to output *)
         while not eoln(afile) do begin
            read(afile, c);
            write(output,c);
         end;
         readln(afile);

         write(output,separator);

         (* now copy a line of the bfile to output *)
         while not eoln(bfile) do begin
            read(bfile, c);
            write(output,c);
         end;
         readln(bfile);

         writeln(output);

      end
   end;

end;
(* end module pst.themain *)

begin
   themain(afile, bfile);
1: end.










