program same (a,b,output);
(* same : identity of two files
britta swebilius singer and thomas d. schneider 1984

module libraries required: delman *)

const
(* begin module version *)
version = 1.12; (* of same.p 1985 apr 20
      date of origin: 1984 march 26 *)
(* end module version *)

(* begin module describe.same *)
(*
name
      same: counts the number of lines that are identical in two files

synopsis
      same(a: in, b: in, output: out)

files
      a: any file
      b: a file to be compared to file a
      output: messages to the user.

description
      same counts the number of lines that are identical in two files,
      a and b.  if the files are identical up to the end of one file,
      but the other file continues, same counts the identical lines and
      tell the user which file is shorter.   no lines are examined after
      the first line that differs between the two files.
         blanks at the ends of lines are ignored.

see also
      merge.p

authors
      britta swebilius singer and thomas d. schneider

bugs
      none known

*)
(* end module describe.same *)

var
(* begin module same.var *)
(* variables of same program *)
      a,b (* the files to be compared *) : text;
      counter (* counts identical lines *) : integer;
      match (* true if a and b are identical *) : boolean;
      empty (* true if either file is empty *) : boolean;
(* end module same.var *)

(* begin module complines *)
function complines(var a,b: text): boolean;
(* compare one line from each of files a and b starting at the
   current file positions.  both lines are compared up to and including
   the end-of-line mark (i.e., they are both readln-ed).  return true if
   the lines are identical and false if they are not.  ignore trailing
   blanks.  this is done by letting chara or charb assume the
   value of a blank when eoln is true.  when both lines are eoln,
   the next line is inspected.  if one line is at end-of-file and
   the other is not, 'false' is returned.  if both lines are at
   eof, 'true' is returned. *)
var
      chara,charb: char; (*  characters in files a and b *)
      done: boolean; (* stop looking for identity *)
begin
      if eof(a) and eof(b)
         then complines := true
         else if eof(a) or eof(b)
            then complines := false
            else begin (* neither file is eof. *)
               done := false;
               while not done do begin
                  if eoln(a) then chara := ' '
                             else read(a,chara);

                  if eoln(b) then charb := ' '
                             else read(b,charb);

                  if eoln(a) and eoln(b)
                     then begin
                        done := true;
                        complines := true
                     end
                     else if chara <> charb then begin
                        done := true;
                        complines := false
                     end;
               end;
               readln(a);
               readln(b);
            end
end; (* complines *)
(* end module complines *)

begin (* same program *)
      writeln(output,' same ',version:4:2);
      reset(a);
      reset(b);
      match := false;  (* we want this to be false unless *)
                       (* we find a set of identical lines *)
      counter := 0; (* so far, no lines are identical *)
      empty := false;  (* if either or both of the files is *)
                       (* empty, this will become true *)
      if eof(a) or eof(b) then empty := true; (* lets us know if one or *)
                                      (* both files are empty to start with *)
                                      (* so that these can be dealt with *)
                                      (* differently from the ones that get *)
                                      (* to be eof through the action of *)
                                      (* complines. *)
      if eof(a) and eof(b)
         then writeln(output,' both files are empty')
         else if eof(a) then writeln(output,' file a is empty')
         else if eof(b) then writeln(output,' file b is empty')
         else begin
            match := true;  (* we want to start this loop with match = true *)
                     (* if we do not get into this loop, match is still false *)
            while match and not (eof(a) or eof(b))
               do begin
                  match := complines(a,b);
                  if match then counter := counter + 1
               end
         end;
      if match
         then begin
            if eof(a) and eof(b) then begin
               writeln(output,' files a and b are identical.');
               writeln(output,' each contains ',counter:1,' line(s).')
            end
            else begin
               if eof(a) and not eof(b)
                  then writeln(output,' file a is shorter than file b')
                  else if eof(b) and not eof(a)
                     then writeln(output,' file b is shorter than file a');
               writeln(output,' and ends at line ',counter:1,
                       '.  up to and including line ',counter:1,',');
               writeln(output,' the two files are identical.')
            end
         end
         else begin
            if (counter > 0)
               then begin
                  writeln(output,
                       ' files a and b are different from each other ');
                  writeln(output,' after the first ',counter:1,
                           ' lines, which are identical.')
               end
            else if (counter = 0) and not empty
               then  writeln(output,' files a and b differ from each other',
                            ' in the first line.')
         end
end. (* same program *)
