program shrink(input,shrinkp,output);
(* shrink: reduce size of postscript graphics

  Tom Schneider
  Frederick, Maryland  21701-1013
  toms@alum.mit.edu

  1989
 *)

label 1; (* end of program *)

const
(* begin module version *)
version = 1.01; (* of shrink.p 1989 November 14
origin 1989 November 14 *)
(* end module version *)

(* begin module describe.shrink *)
(*
name
   shrink: reduce size of postscript graphics

synopsis
   shrink(input: in, output: out)

files
   input:  A PostScript program, containing a translate command.
   shrinkp: Parameter file.  The first line contains the scale factor.
   output: A copy of the input with the scale instructions.  A scale
      command is placed immediately after the translate command, so that
      the shrinking occurs toward the zero of the image.

description
   One often wants to run rsgra to look at a large region of aligned sequence,
   but the normal output won't fit on a page.  By passing the PostScript
   file through this program, one can scale the graphics to something
   that fits on a page.

examples
   0.5  would reduce the size of the image by a factor of 2.
   Note: the 0 is necessary for most Pascal compilers.

documentation

see also
   rsgra.p

author
   Thomas Dana Schneider

bugs
   The program is very specific in what it does.

technical notes

*)
(* end module describe.shrink *)

(* begin module interact.const *)
      maxstring = 150; (* the maximum string *)
(* end module interact.const version = 'prgmod 4.05  89 Aug 28 tds'; *)

(* begin module dops.filler.const *)
      fillermax = 20; (* the size of the filler array for a string *)
(* end module dops.filler.const *)

type
(* begin module interact.type *)
      string = record (* a string of characters *)
         letters: array[1..maxstring] of char; (* the letters in the string *)
         length: integer; (* the number of characters in the string *)
         current: integer; (* the letter we are working on *)
      end;
(* end module interact.type version = 'prgmod 4.05  89 Aug 28 tds'; *)

(* begin module trigger.type *)
      trigger = record (* an object to be searched for *)
         seek: string; (* the characters looked for *)
         state: integer; (* how close to triggering we are *)
         skip: boolean; (* trigger not found- skip the line *)
         found: boolean (* the trigger was found *)
      end;
(* end module trigger.type version = 'prgmod 4.05  89 Aug 28 tds'; *)

(* begin module filler.type *)
      (* the following is an array used to fill a string.
      it is convenient to have it much shorter than the maxstring, so that
      it is easy to fill the string using procedure fillstring.
      the user must declare the value of constant fillermax. *)
      filler = packed array[1..fillermax] of char;
(* end module filler.type version = 'prgmod 4.05  89 Aug 28 tds'; *)

var
   shrinkp: text; (* parameters *)

(* begin module halt *)
procedure halt;
(* stop the program.  the procedure performs a goto to the end of the
   program.  you must have a label:
      label 1;
   declared, and also the end of the program must have this label:
      1: end.
   examples are in the module libraries.
   this is the only goto in the delila system. *)
begin
      writeln(output,' program halt.');
      goto 1
end;
(* end module halt version = 'prgmod 4.05  89 Aug 28 tds'; *)

(* begin module interact.clearstring *)
procedure clearstring(var ribbon: string);
(* empty the string *)
var   index: integer; (* to the ribbon *)
begin (* clearstring *)
      with ribbon do begin
         for index := 1 to maxstring do letters[index] := ' ';
         length := 0;
         current := 0;
      end
end; (* clearstring *)
(* end module interact.clearstring version = 'prgmod 4.05  89 Aug 28 tds'; *)

(* begin module trigger.proc *)
(* this module allows one to scan a series of characters, as from
an array or a file, and to "trigger" or detect a simple string
in the series.  the advantage of the trigger is that several triggers
can "observe" a stream of characters at once, each looking for a
different thing.
some other modules required: interact.const, interact.type *)

procedure resettrigger(var t: trigger);
(* reset the trigger to ground state *)
begin (* resettrigger *)
      with t do begin
         state := 0;
         skip := false;
         found := false
      end
end; (* resettrigger *)

procedure testfortrigger(ch: char; var t: trigger);
(* look at the character ch.
   if it is part of the trigger (at the current trigger state),
       then the trigger state goes higher.
   if it is not part of the trigger then the trigger state is reset,
      skip is true and one should skip onward to find the trigger.
   if the trigger is found, found is true. *)
begin (* testfortrigger *)
      with t do begin
         state := succ(state);
(*         if debugging then begin
            writestring(list,seek);
            writeln(list,'testfortrigger seek.letters[',state:1,']:',
                           seek.letters[state],' ch:',ch);
         end;*)
         if seek.letters[state] = ch
         then begin
            skip := false;
            if state = seek.length then found := true
                                   else found := false
         end
         else begin (* reset trigger *)
            state := 0;
            skip := true;
            found := false
         end
      end
end; (* testfortrigger *)
(* end module trigger.proc version = 'prgmod 4.05  89 Aug 28 tds'; *)

(* begin module filler.fillstring *)
procedure fillstring(var s: string; a: filler);
(* this procedure makes it reasonably easy to fill the string s with
characters.  one calls the procedure as: *)
(*                           1         2         3         4         5 *)
(*                  12345678901234567890123456789012345678901234567890 *)
(*   fillstring(s, 'this-is-the-string                                ');
the two comments make it easy to line the characters up. also, for this
example, it was assumed that the length of filler as defined by the
constant fillermax was 50. *)
var
      length: integer; (* of the string without trailing blanks *)
      index: integer; (* of s *)
begin (* fillstring *)
      clearstring(s);
      length := fillermax;
      while (length > 1) and (a[length] = ' ') do length := pred(length);
      if (length = 1) and (a[length] = ' ') then begin
         writeln(output, 'fillstring: the string is empty');
         halt
      end;

      for index := 1 to length do s.letters[index] := a[index];
      s.length := length;
      s.current := 1
end; (* fillstring *)
(* end module filler.fillstring version = 'prgmod 4.05  89 Aug 28 tds'; *)

(* begin module filler.filltrigger *)
procedure filltrigger(var t: trigger;
                      a: filler);
(* fill the trigger t *)
begin (* filltrigger *)
         fillstring(t.seek,a)
end; (* fillstring *)
(* end module filler.filltrigger version = 'prgmod 4.05  89 Aug 28 tds'; *)

(* begin module copyaline *)
procedure copyaline(var fin, fout: text);
(* copy a line from file fin to file fout *)
begin (* copyaline *)
      while not eoln(fin) do begin
         fout^ := fin^;
         put(fout);
         get(fin)
      end;
      readln(fin);
      writeln(fout);
end; (* copyaline *)
(* end module copyaline version = 'prgmod 4.05  89 Aug 28 tds'; *)

(* begin module shrink.themain *)
procedure themain(var fin,shrinkp,fout: text);
(* the main procedure of the program *)
var
   c: char; (* a character in fin *)
   factor: real; (* the fraction by which to scale the image *)
   gotit: boolean; (* true if the trigger was found *)
   translate: trigger; (* the word 'translate' to be found *)
begin
   writeln(fout,'% shrink ',version:4:2);
   reset(shrinkp);
   readln(shrinkp,factor);

(*                                       1         2 *)
(*                              12345678901234567890 *)
filltrigger(translate         ,'translate           ');
resettrigger(translate);

   gotit := false;

   (* copy the first part of the file *)
   while (not eof(fin)) and (not gotit) do begin
      while not eoln(fin) do begin
         read(fin,c);
         write(fout,c);
         testfortrigger(c,translate);
         if translate.found then gotit := true
      end;
      readln(fin);
      writeln(fout);
   end;

   if gotit then writeln(output,factor:10:4,' ',factor:10:4,' scale');

   (* simply copy the rest of the file *)
   while not eof(fin) do copyaline(fin,fout);

end;
(* end module shrink.themain *)

begin
   themain(input,shrinkp,output);
1: end.
