program divide(histog, surprisalp, xyin, output);
(* surprisal: make the surprisal curve for a histogram

  Tom Schneider
  permanent email: toms@alum.mit.edu
  toms@alum.mit.edu
  https://alum.mit.edu/www/toms/


 *)

label 1; (* end of program *)

const
(* begin module version *)
version = 1.00; (* of surprisal.p 2000 January 10
origin 2000 January 10 *)
updateversion = 1.00; (* defines lowest acceptable current parameter file *)
(* end module version *)

(* begin module describe.divide *)
(*
name
   surprisal: make the surprisal curve for a histogram

synopsis
   surprisal(histog: in, surprisalp: in, xyin: out, output: out)

files
   histog:  position and number

   xyin: Columns generated by the surprisal program:
      1: position
      2: -log2(frequency), surprisal in bits
      3: frequency
      4: original count in this bin

   surprisalp:  parameters to control the program.  The file must contain the
      following parameters, one per line:

      parameterversion: The version number of the program.  This allows the
         user to be warned if an old parameter file is used.

   output: messages to the user

description

   Convert a histogram into a surprisal curve by finding the sum of the
   values in the histogram, converting the histogram to frequencies and
   taking the negative log base two of each frequency.

examples

documentation

see also
   {to generate the histog, use:} genhis.p
   {to plot the result, use:} xyplo.p

author
   Thomas Dana Schneider

bugs

technical notes

*)
(* end module describe.surprisal *)

var
   histog, (* file used by this program *)
   surprisalp, (* file used by this program *)
   xyin: text; (* file used by this program *)

(* begin module halt *)
procedure halt;
(* stop the program.  the procedure performs a goto to the end of the
   program.  you must have a label:
      label 1;
   declared, and also the end of the program must have this label:
      1: end.
   examples are in the module libraries.
   this is the only goto in the delila system. *)
begin
      writeln(output,' program halt.');
      goto 1
end;
(* end module halt version = 'delmod 6.16 84 mar 12 tds/gds'; *)

(* begin module copyaline *)
procedure copyaline(var fin, fout: text);
(* copy a line from file fin to file fout *)
begin (* copyaline *)
      while not eoln(fin) do begin
         fout^ := fin^;
         put(fout);
         get(fin)
      end;
      readln(fin);
      writeln(fout);
end; (* copyaline *)
(* end module copyaline version = 4.39; (@ of prgmod.p 1999 November 28 *)

(* begin module surprisal.themain *)
procedure themain(var histog, xyin: text);
(* the main procedure of the program *)
const
   wid = 10; (* width of numbers *)
   dec = 8; (* decimals of numbers *)
var
   numerator: integer; (* number to be divided *)
   ln2: real; (* log base e of 2 *)
   parameterversion: real; (* parameter version number *)
   position: integer; (* position in histogram *)
   total: integer; (* total of the numbers *)
begin
   writeln(output,'surprisal ',version:4:2);

   reset(surprisalp);
   readln(surprisalp, parameterversion);
   if parameterversion < updateversion then begin
      writeln(output, 'You have an old parameter file!');
      halt
   end;

   rewrite(xyin);
   writeln(xyin,'* surprisal ',version:4:2);
   reset(histog);

   total := 0;
   while not eof(histog) do begin
      if eoln(histog)
      then readln(histog)
      else if histog^ = '*' 
           then readln(histog)
           else begin
              readln(histog, position, numerator);
              total := total + numerator
           end
   end;

   reset(histog);
   ln2 := ln(2.0);

   writeln(xyin,'*');
   writeln(xyin,'* columns generated by the surprisal program:');
   writeln(xyin,'* 1: position');
   writeln(xyin,'* 2: -log2(frequency), surprisal in bits');
   writeln(xyin,'* 3: frequency');
   writeln(xyin,'* 4: original count in this bin');
   writeln(xyin);

   while not eof(histog) do begin
      if eoln(histog)
      then readln(histog)
      else if histog^ = '*' 
           then copyaline(histog, xyin)
           else begin
              readln(histog, position, numerator);
              writeln(xyin, position:1,
                            ' ', -ln(numerator/total)/ln2:wid:dec,
                            ' ', (numerator/total):wid:dec,
                            ' ', numerator:1);
           end
   end;

end;
(* end module surprisal.themain *)

begin
   themain(histog, xyin);
1: end.
