program timep2c(output); 
(* timep2c: test time on the p2c translator/compiler

  Thomas D. Schneider, Ph.D.
  toms@alum.mit.edu
  https://alum.mit.edu/www/toms 

*) 
  
label 1;  (* the end of the program *)
  
const 
(* begin module version *)
   version = 1.33; (* of timep2c.p 2017 Feb 21
2017 Feb 21, 1.33: code bug detected by Schindler Karl-Michael
                   <karl-michael.schindler@web.de>
   at the end of getdatetime was
   for index:=18 to datetimearraylength do adatetime[index+2]:=' '; 
   but with datetimearraylength = 19;
   this violates the array bound!  Delete the line as it does nothing.
2016 Aug 29, 1.32: remove lecb
2016 Jan 24, 1.31: correct:
    sed -i.bak 's|1..11|1..datetimearraylength|g'     delmod.p
2002 Oct  9, 1.30: make link to module.p
2001 Jan  2, 1.29: upgrade readdatetime
2000 Oct 11, 1.26: upgrade documentation and timeseed
2000 Oct  9, 1.25: remove alpha and namelength
2000 Oct  3, 1.24: adjust getdatetime documentation
2000 Oct  2, 1.23: fix version number output
2000 Jul 25, 1.22: upgrade documentation
2000 Jul 23, 1.20: change name from cdatemod to timep2c
1999 Dec 13, 1.19: last previous change
1999 Mar 4: upgraded to new datetimearray
origin from unixmod 1996 july 25 *) 
(* end module version *)
  
(* begin module describe.timep2c *)
(*
name
   timep2c: test time on the p2c translator/compiler
  
synopsis
   timep2c(output: out) 
  
files 
   output: where the date and time will appear.  
  
description 

   The time function returned by various computers is different.  Show the
   time function to figure out how to modify the getdatetime procedure.
 
   This program contains routines for obtaining the date and time for Delila
   programs compiled with the p2c translater and the gcc compiler.

examples 
 
   Times should look like this:
       1980/06/09 18:49:11
         ye mo da ho mi se

documentation

   moddef, delman.describe.module

see also

   {The module program, used to transfer the modules is:} module.p

   delman.describe.delmod, moddef, delman.describe.module delmod.p,
   y2k.p

   {Related date/time programs:}
   timegpc.p
   timesun.p

   {General discussion on compiling Delila programs:} 
   https://alum.mit.edu/www/toms/delila.html#How.To.Compile 
  
author

   Tom Schneider
  
bugs

   none known  
  
technical notes 

*)
(* end module describe.timep2c *)

(* begin module package.datetime *) 
(* ************************************************************************ *)
(* begin module getdatetime *)
(* end module getdatetime *)
  
(* begin module readdatetime *) 
(* end module readdatetime *) 
  
(* begin module writedatetime *)
(* end module writedatetime *)
  
(* begin module timeseed *)
(* end module timeseed *)

(*[[*)
(* begin module limitdate *)
(* end module limitdate *)
(*]]*)
  
(* ************************************************************************ *)
(* end module package.datetime *) 

(* begin module datetime.const *) 
   datetimearraylength = 19; (* length of dataarray for dates,
       It is just long enough to include the 4 digit year - solving the
       year 2000 problem:
1980/06/09 18:49:11
123456789 123456789 
         1         2
*)
(* end module datetime.const version = 'delmod 6.74 97 Mar 16 tds/gds' *)
  
type
(* begin module datetime.type *)  
   (* array for dates *)
   datetimearray = packed array[1..datetimearraylength] of char;
(* end module datetime.type version = 'delmod 6.74 97 Mar 16 tds/gds' *)

var 
(* begin module timep2c.var *) 
   (* this is a global variable for use in the running of timep2cs *) 
   adatetime: datetimearray;
(* end module timep2c.var *) 
  
(* begin module timep2c.halt *)  
procedure halt; 
 (* halt the program and notify the user. 
 the module name is not the standard one so that this procedure will not be 
 transfered when timep2c is used as a modlib. this avoids the possibility
 of transfering an obsolete halt module into a main module library
 unknowingly. *)
begin 
      writeln(output,' program halt.'); 
      goto 1
end;
(* end module timep2c.halt *)  
  
(* begin module testdatetimep2c *)
procedure testdatetimep2c;
(* Give the values of the date and time arrays returned.
This is specific to the p2c compiler *)
var
   adate, atime: packed array[1..datetimearraylength] of char;
begin
   writeln(output, 'Note: the following routines are specific to the p2c',
         ' compiler: ');
   writeln(output,'the raw date and time are:');
   date(adate);
   time(atime);
   writeln(output,'adate[',adate,']');
   writeln(output,'atime[',atime,'] (note extra random characters)');
end;
(* end module testdatetimep2c *)

(* begin module getdatetime *)
procedure getdatetime(var adatetime: datetimearray);
(* Get the date and time into a single array from the system clock.
   adatetime contains the date: 
       1980/06/09 18:49:11
         ye mo da ho mi se
(year, month, day, hour, minute, second)
This version works after translation of the pascal by p2c
to C and then compiling with gcc. *) 
var 
   adate, atime: packed array[1..datetimearraylength] of char;
{
   adate, atime: alfa; (* ie, packed array[1..10] of char; *)
This old method won't work, since the last digit gets cut off!
}
   month: packed array[1..3] of char;
   index: integer; (* index for times *) 
begin 
(*
                1 
      12345678901
adate[13-DEC-1999]
atime[17:39:44.00]
*)

   date(adate);
   time(atime);

{
writeln(output,'br: adate[',adate,'] atime[',atime,']');
}

   (* transfer the year *)
   for index:=1 to 4 do adatetime[index]:=adate[index+7];
   adatetime[5]:='/';
   for index:=4 to 6 do month[index-3]:=adate[index];
        if month='JAN' then begin adatetime[6]:='0'; adatetime[7]:='1' end 
   else if month='FEB' then begin adatetime[6]:='0'; adatetime[7]:='2' end 
   else if month='MAR' then begin adatetime[6]:='0'; adatetime[7]:='3' end 
   else if month='APR' then begin adatetime[6]:='0'; adatetime[7]:='4' end 
   else if month='MAY' then begin adatetime[6]:='0'; adatetime[7]:='5' end 
   else if month='JUN' then begin adatetime[6]:='0'; adatetime[7]:='6' end 
   else if month='JUL' then begin adatetime[6]:='0'; adatetime[7]:='7' end 
   else if month='AUG' then begin adatetime[6]:='0'; adatetime[7]:='8' end 
   else if month='SEP' then begin adatetime[6]:='0'; adatetime[7]:='9' end 
   else if month='OCT' then begin adatetime[6]:='1'; adatetime[7]:='0' end 
   else if month='NOV' then begin adatetime[6]:='1'; adatetime[7]:='1' end 
   else if month='DEC' then begin adatetime[6]:='1'; adatetime[7]:='2' end;
   adatetime[8]:='/';
   for index:=7 to 8 do adatetime[index+2]:=adate[index-6];

   (* replace blanks with spaces in dates *)
   if adatetime[6]=' ' then adatetime[6]:='0';
   if adatetime[9]=' ' then adatetime[9]:='0';

   adatetime[9+2]:=' ';
   for index:=10 to 17 do adatetime[index+2]:=atime[index-9];
end;
(* end module getdatetime *)
  
(* begin module readdatetime *) 
procedure readdatetime (var thefile: text; var adatetime: datetimearray);
(* read the date and time from the file.  It must have this format:
123456789 123456789
         1         
1980/06/09 18:49:11
 *)
(*
2001 Jan 2: thefile is not written to!  Force 4 digit years.
2000 Oct 11: upgraded so that the p2c compiler does not object
to writing out the adatetime; added checks for the date.
*)
var
   index: integer; (* to the udatetime *)
   (* the following is an unpacked date time array, to avoid reading
   into a packed array.  reading into a packed array is not transportable *)
   udatetime: array[1..datetimearraylength] of char;
begin
   for index:=1 to datetimearraylength do read(thefile,udatetime[index]);
   pack(udatetime, 1, adatetime);
   if (adatetime[3]='/') and (adatetime[12]=':') then begin
      writeln(output,'You have an old datetime (only 2 year digits): ');
      for index:=1 to datetimearraylength do write(output,adatetime[index]);
      writeln(output);
      writeln(output,'Convert your database to 4 digit years.');
      halt;
   end;
   (* check the adatetime format.  Note that further checks for the
      other positions in the array could be done to be sure that
      they are numbers.  But this should be pretty good. *)
   if (adatetime[ 5]<>'/') or
      (adatetime[ 8]<>'/') or
      (adatetime[14]<>':') or
      (adatetime[17]<>':')
   then begin
      writeln(output,'readdatetime: bad date time read:');
      for index:=1 to datetimearraylength do write(output,adatetime[index]);
      writeln(output);
      halt
   end;
end;
(* end module readdatetime *)
  
(* begin module writedatetime *)
procedure writedatetime(var thefile: text; adatetime: datetimearray);
(* expand the date and time out and print in the file *)
var
      index: integer; (* index of datetime *)
begin
      for index:=1 to datetimearraylength do write(thefile,adatetime[index])
end;
(* end module writedatetime version = 'delmod 6.74 97 Mar 16 tds/gds' *)

(* begin module timeseed *)
(* Read the computer date and time.  Reverse the order of the digits and put
a decimal point in front.  This gives a fraction between zero and one that
varies quite quickly, and is always unique (if the computer has sufficient
accuracy).  It is to be used as a seed to a random number generator.  This
has the nice property that the seed changes every second and does not repeat
for thousands of years!  *)
procedure addtoseed(var seed, power: real; c: char);
(* add the digit represented by c to the seed at the power position *)
var
      n: integer; (* the character represented by c *)
begin (* addtoseed *)
      power := power/10;
{
writeln(output,'addtoseed, c = ',c);
writeln(output,'addtoseed, ord(c) = ',ord(c));
}
   n := ord(c) - ord('0');
   if (n < 0) or (n > 9) then begin
      writeln(output,'timeseed: error in datetime');
      writeln(output,'it contains "',c,'" which is not a number.');
      writeln(output,'The getdatetime routine must be fixed.');
      halt;
   end;
   seed := seed + power*n
end; (* addtoseed *)

procedure makeseed(adatetime: datetimearray; var seed: real);
(* convert adatetime to a real number in seed, reversed order
Here is the standard adatetime format:
123456789 123456789
         1         
1980/06/09 18:49:11
*)
var
   power: real; (* a digit of the seed such as 0.01 *)
begin
   seed := 0.0;
   power := 1.0;
   addtoseed(seed, power, adatetime[19]);
   addtoseed(seed, power, adatetime[18]);
   (* : *)
   addtoseed(seed, power, adatetime[16]);
   addtoseed(seed, power, adatetime[15]);
   (* : *)
   addtoseed(seed, power, adatetime[13]);
   addtoseed(seed, power, adatetime[12]);
   (*   *)
   addtoseed(seed, power, adatetime[10]);
   addtoseed(seed, power, adatetime[ 9]);
   (* / *)
   addtoseed(seed, power, adatetime[ 7]);
   addtoseed(seed, power, adatetime[ 6]);
   (* / *)
   addtoseed(seed, power, adatetime[ 4]);
   addtoseed(seed, power, adatetime[ 3]);
   addtoseed(seed, power, adatetime[ 2]);
   addtoseed(seed, power, adatetime[ 1]);
end;

procedure orderseed(adatetime: datetimearray; var seed: real);
(* convert adatetime to a real number in seed, normal order *)
var
   power: real; (* a digit of the seed such as 0.01 *)
begin
   seed := 0.0;
   power := 1.0;
   addtoseed(seed, power, adatetime[ 3]);
   addtoseed(seed, power, adatetime[ 4]);
   addtoseed(seed, power, adatetime[ 6]);
   addtoseed(seed, power, adatetime[ 7]);
   (* / *)
   addtoseed(seed, power, adatetime[ 9]);
   addtoseed(seed, power, adatetime[10]);
   (* / *)
   addtoseed(seed, power, adatetime[12]);
   addtoseed(seed, power, adatetime[13]);
   (*   *)
   addtoseed(seed, power, adatetime[15]);
   addtoseed(seed, power, adatetime[16]);
   (* : *)
   addtoseed(seed, power, adatetime[18]);
   addtoseed(seed, power, adatetime[19]);
end;

procedure timeseed(var seed: real);
(* read the computer date and time.  reverse the order of the digits
and put a decimal point in front.  this gives a fraction between
zero and one that varies quite quickly, and is always unique (if the
computer has sufficient accuracy).  it is to be used as a seed to
a random number generator. *)
var
   adatetime: datetimearray; (* a date and time *)
begin (* timeseed *)
   getdatetime(adatetime);
{
writeln(output,'timeseed: adatetime: ',adatetime);
}
   makeseed(adatetime, seed);
end; (* timeseed *)
(* end module timeseed version = 1.01; (@ of testtime.p 1997 Jan 11 *)

(*[[*)
(* begin module limitdate *)
procedure limitdate(a,b,c,d: char; limitdatetime: datetimearray);
(* test whether the current time is before the limit.
If it is later, halt the program *)
var
   adatetime: datetimearray; (* a date and time *)
   Dday: real; (* the critical day *)
   now: real;  (* this very moment *)
begin
   getdatetime(adatetime);
{
writeln(output,'adatetime:',adatetime);
writeln(output,'adatetime[1]: ', adatetime[1]);
writeln(output,'adatetime[2]: ', adatetime[2]);
writeln(output,'adatetime[3]: ', adatetime[3]);
writeln(output,'adatetime[4]: ', adatetime[4]);
writeln(output,'adatetime[5]: ', adatetime[5]);
}

   orderseed(adatetime, now);

   if
      (limitdatetime[1] <> ' ') or
      (limitdatetime[2] <> ' ') or
      (limitdatetime[3] <> ' ') or
      (limitdatetime[4] <> ' ') then halt;

   limitdatetime[1] := a;
   limitdatetime[2] := b;
   limitdatetime[3] := c;
   limitdatetime[4] := d;

   orderseed(limitdatetime, Dday);

{
   writeln(output,'now:  ',now:20:10);
   writeln(output,'Dday: ',Dday:20:10);
}
   if now > Dday then begin
       writeln(output,'This program expired on ',limitdatetime);
       writeln(output,'See: https://alum.mit.edu/www/toms/walker/contacts.html');
       halt
   end
end;
(* end module limitdate *)
(*]]*)

(* begin module demo.time *)
procedure demotime(var fout: text);
(* write the time to file fout *)
var   dateandtime: datetimearray; (* the date and time *)
      seed: real; (* a seed for a random number generater, made
         from the date and time written backwards *)
begin (* demotime *)
   getdatetime(dateandtime);
   writeln(fout);
   writeln(fout,'The date and time is: ');
   writedatetime(fout, dateandtime);
   writeln(fout,' <- This should be the current time and date.');
   write  (fout,'1980/06/09 18:49:11');
   writeln(fout,' <- Times and dates should look like this');
   writeln(fout,'year mo da ho mi se <- with parts in these positions');
   writeln(fout);
 
   timeseed(seed);
   writeln(fout,'A timeseed is ',seed:16:14);
   writeln(fout,'Timeseeds can be used to start random number generators.')
end; (* demotime *)
(* end module demo.time version = 7.40; {of delmod.p 2000 Feb 18} *)
  
begin  (* timep2c *) 
 writeln(output,'timep2c ',version:4:2);

 (* this test is specific: *)
 testdatetimep2c;

 (* this test is general: *)
 demotime(output);
1:end.   (* timep2c *) 
