program ww(input,output);
(* ww: word wrap 
   Tom Schneider *)

const
(* begin module version *)
version = 1.03; (* of ww.p 1996 February 19
origin 1988 aug 22 *)
(* end module version *)

(* begin module describe.ww *)
(*
name
   ww: word wrap 

synopsis
   ww(input: in, output: out)

files
   input:  text to be wrapped
   output: wrapped text

description
   This Pascal program takes ASCII text and filters it.  Lines longer than the
constant maxline are altered by replacing the first space after position
maxline with a carriage return.  This has the effect of wrapping the lines
between 'words'.
   The original purpose was to get around a design flaw in another program.
The program fig produces graphics for X and NeWS windows.  The graphics
is converted to PostScript by another program, f2ps.  Unfortunately f2ps
was poorly designed: the PostScript produced has many lines longer than
70 characters.  When this PostScript code is sent to the (latest as
of 1988) Apple NTX LaserWriterII, the printer dies.  By running this filter,
the problem is bypassed.  Moral: never make lines longer than 80 characters!

author
   Thomas Dana Schneider

bugs
   the constant maxline is fixed at compile time, of course.

*)
(* end module describe.ww *)

    maxline = 70; (* maximum width of output lines *)

var c: char; (* a character in the input *)
    n: integer; (* number of characters written to output so far *)
    waitforspace: boolean; (* true if waiting for the next space to appear *)
begin
  while not eof(input) do begin
    n := 0; (* no characters written out so far on this line *)
    waitforspace := false;
    while not eoln(input) do begin
      if n > maxline then waitforspace := true;

      if waitforspace and (input^ = ' ')
      then begin 
         writeln(output); (* replace space with carriage return *)
         get(input); (* gobble up the space *)
         n := 0; (* no characters written out so far on this line *)
         waitforspace := false;
      end
      else begin
        write(output,input^);
        get(input);
        n := n + 1;
      end
    end;

    readln(input);
    writeln(output);
  end
end.
