program wwlow(input,output);
(* wwlow: word wrap, low order ASCII
   Tom Schneider *)

const
(* begin module version *)
version = 1.01; (* of wwlow.p 2002 July 10
2002 July 10, 1.01: fix documentation
2002 July 10, 1.00: origin from ww
*)
(* end module version *)

(* begin module describe.wwlow *)
(*
name
   wwlow: word wrap, low order ASCII

synopsis
   wwlow(input: in, output: out)

files
   input:  text to be wrapped
   output: wrapped text

description

   This Pascal program takes ASCII text and filters it.  Lines longer than
the constant maxline are altered by replacing the first space after position
maxline with a carriage return.  This has the effect of wrapping the lines
between 'words'.

   Like the ww program, this program wraps, but it drops all ASCII characters
that are not in the range 32 (space) to 126 (~).  This is useful for cleaning
up wordY or wordIMperfekt files.

examples

documentation

see also
   {word wrap program that does not filter:} ww.p

author
   Thomas Dana Schneider

bugs
   The constant maxline is fixed at compile time, of course.

*)
(* end module describe.wwlow *)

    maxline = 70; (* maximum width of output lines *)

var c: char; (* a character in the input *)
    n: integer; (* number of characters written to output so far *)
    waitforspace: boolean; (* true if waiting for the next space to appear *)
begin
  while not eof(input) do begin
    n := 0; (* no characters written out so far on this line *)
    waitforspace := false;
    while not eoln(input) do begin
      if n > maxline then waitforspace := true;

      if waitforspace and (input^ = ' ')
      then begin 
         writeln(output); (* replace space with carriage return *)
         get(input); (* gobble up the space *)
         n := 0; (* no characters written out so far on this line *)
         waitforspace := false;
      end
      else begin
        if (ord(input^) >=  32) and (* space *)
           (ord(input^) <= 126) and (* ~ *)
           (not (ord(input^) = 13))  (* CR *)
{
Not sure about this
           (not (ord(input^) = 12)  (* LF *)
}
        then write(output,input^);

(* man ascii gives:

       0 NUL   1 SOH    2 STX    3 ETX    4 EOT    5 ENQ    6 ACK    7 BEL
       8 BS    9 HT    10 NL    11 VT    12 NP    13 CR    14 SO    15 SI
      16 DLE  17 DC1   18 DC2   19 DC3   20 DC4   21 NAK   22 SYN   23 ETB
      24 CAN  25 EM    26 SUB   27 ESC   28 FS    29 GS    30 RS    31 US
      32 SP   33 !     34 "     35 #     36 $     37 %     38 &     39 '
      40 (    41 )     42 *     43 +     44 ,     45 -     46 .     47 /
      48 0    49 1     50 2     51 3     52 4     53 5     54 6     55 7
      56 8    57 9     58 :     59 ;     60 <     61 =     62 >     63 ?
      64 @    65 A     66 B     67 C     68 D     69 E     70 F     71 G
      72 H    73 I     74 J     75 K     76 L     77 M     78 N     79 O
      80 P    81 Q     82 R     83 S     84 T     85 U     86 V     87 W
      88 X    89 Y     90 Z     91 [     92 \     93 ]     94 ^     95 _
      96 `    97 a     98 b     99 c    100 d    101 e    102 f    103 g
     104 h   105 i    106 j    107 k    108 l    109 m    110 n    111 o
     112 p   113 q    114 r    115 s    116 t    117 u    118 v    119 w
     120 x   121 y    122 z    123 {    124 |    125 }    126 ~    127 DEL

*)
        get(input);
        n := n + 1;
      end
    end;

    readln(input);
    writeln(output);
  end
end.
